this.name = "Repair system";
this.author = "Thargoid";
this.copyright = "Creative Commons Attribution - Non-Commercial - Share Alike 3.0 license with clauses - see readme.txt.";
this.description = "Damage Control system based on repair bots";
this.version = "2.10";
"use strict";

this.startUp = function()
	{ 
	if(!missionVariables.repairCounter) { missionVariables.repairCounter = 10; } 
	this.delayArray = [];
	}

this.playerBoughtEquipment = function(equipment)
	{
	switch(equipment)
		{
		case "EQ_REPAIRBOTS_CONTROLLER":
			{
			missionVariables.repairCounter = 10;
			break;
			}
		case "EQ_REPAIRBOTS_RECHARGE":
			{
			player.ship.removeEquipment("EQ_REPAIRBOTS_RECHARGE");
			missionVariables.repairCounter = 10;
			break;
			}
		}
	}

this.playerBoughtNewShip = function(ship)
	{
	if(player.ship.equipmentStatus("EQ_REPAIRBOTS_CONTROLLER") === "EQUIPMENT_OK" || player.ship.equipmentStatus("EQ_REPAIRBOTS_CONTROLLER") === "EQUIPMENT_DAMAGED")
		{ missionVariables.repairCounter = 10; }
	else
		{ missionVariables.repairCounter = 0; }
		
	}
	
	
this.guiScreenWillChange = function(to, from)
	{
	if(to === "GUI_SCREEN_MANIFEST")
		{
		switch(player.ship.equipmentStatus("EQ_REPAIRBOTS_CONTROLLER"))
			{
			case "EQUIPMENT_OK":
				{
				if(missionVariables.repairCounter > 0)
					{ this.repairDisplay = "Remaining repair system charges - " + missionVariables.repairCounter;}
				else
					{ this.repairDisplay = "Repair system offline - recharge required"; }				
				mission.setInstructions(this.repairDisplay); 
				break;
				}
		
			case "EQUIPMENT_DAMAGED":
				{
				mission.setInstructions("Repair system offline - damaged");
				break;
				}			
			
			case "EQUIPMENT_UNAVAILABLE":
			case "EQUIPMENT_UNKNOWN":
				{
				mission.setInstructions(null);
				break;
				}
			}
		}	
	}	
	
this.equipmentDamaged = function(equipment)
	{
	if(player.ship.equipmentStatus("EQ_REPAIRBOTS_CONTROLLER") === "EQUIPMENT_UNAVAILABLE") 
		{ return; }
	this.delayArray.push(equipment);
	if(!this.equipmentDelay)
		{ this.equipmentDelay = new Timer(this, this.checkEquipment, 0.5); }
	}

this.checkEquipment = function()
	{
	if(!this.delayArray || this.delayArray.length === 0)	{ return; }
	var equipment = this.delayArray.pop();
	if(player.ship.equipmentStatus(equipment) !== "EQUIPMENT_DAMAGED") { return; }
	
	if (equipment === "EQ_REPAIRBOTS_CONTROLLER")
		{
		if(this.damageControlTimer && this.damageControlTimer.isRunning)
			{
			this.damageControlTimer.stop();
			delete this.damageControlTimer;
			}
		var repairTime = 60 + Math.ceil(Math.random() * 120);
		player.consoleMessage("Repair system entering self-repair mode", 6);
		this.selfRepairTimer = new Timer(this, this.selfRepair, repairTime);	
		}
	else
		{
		if((missionVariables.repairCounter && missionVariables.repairCounter > 0) && (!this.damageControlTimer || !this.damageControlTimer.isRunning))
			{
			missionVariables.repairCounter--;
			var repairTime = 120 + Math.ceil(Math.random() * 180);
			player.consoleMessage("Repair system activated", 6);
			this.damageControlTimer = new Timer(this, this.repairSystems, repairTime);	
			}
		}
	}	

this.selfRepair = function()
	{
	if(player.ship.equipmentStatus("EQ_REPAIRBOTS_CONTROLLER") === "EQUIPMENT_UNAVAILABLE") 
		{ return; }
	
	if(Math.random() < 0.75) // 75% chance of self-repair
		{
		player.ship.setEquipmentStatus("EQ_REPAIRBOTS_CONTROLLER","EQUIPMENT_OK");
		this.checkSystems();
		if((missionVariables.repairCounter && missionVariables.repairCounter > 0) && this.playerDamagedList.length > 0)
			{
			missionVariables.repairCounter--;
			player.consoleMessage("Repair system online and activated", 6);
			var repairTime = 120 + Math.ceil(Math.random() * 180);
			this.damageControlTimer = new Timer(this, this.repairSystems, repairTime);
			}
		else
			{
			if(missionVariables.repairCounter && missionVariables.repairCounter > 0)
				{ player.consoleMessage("Repair system online and in standby mode", 6); }
			else	
				{ mission.setInstructions("Repair system offline - recharge required"); }
			}	
		}
	else
		{
		if(player.ship.equipmentStatus("EQ_REPAIRBOTS_CONTROLLER") === "EQUIPMENT_DAMAGED") // just in case it was repaired in the meantime
			{
			var repairTime = 60 + Math.ceil(Math.random() * 120);
			player.consoleMessage("Repair system offline - self-repair continuing", 6);
			this.selfRepairTimer = new Timer(this, this.selfRepair, repairTime);
			}
		}
	}
	
this.equipmentDestroyed  = function(equipment)
	{
	if(equipment === "EQ_REPAIRBOTS_CONTROLLER")
		{ 
		this.stopTimers(); 
		mission.setInstructions(null); 
		}
	}	
	
this.stopTimers = this.shipWillDockWithStation = this.shipDied = function()
	{
	if(this.damageControlTimer && this.damageControlTimer.isRunning)
		{
		this.damageControlTimer.stop();
		delete this.damageControlTimer;
		}
	if(this.selfRepairTimer && this.selfRepairTimer.isRunning)
		{
		player.ship.setEquipmentStatus("EQ_REPAIRBOTS_CONTROLLER","EQUIPMENT_OK");
		this.selfRepairTimer.stop();
		delete this.selfRepairTimer;
		}	
	}

this.shipWillLaunchFromStation = function()
	{
	if(player.ship.equipmentStatus("EQ_REPAIRBOTS_CONTROLLER") === "EQUIPMENT_OK")
		{
		this.checkSystems();
		if((missionVariables.repairCounter && missionVariables.repairCounter > 0) && this.playerDamagedList.length > 0)
			{
			missionVariables.repairCounter--;
			var repairTime = 120 + Math.ceil(Math.random() * 180);
			player.consoleMessage("Repair system reactivated", 6);
			this.damageControlTimer = new Timer(this, this.repairSystems, repairTime);	
			}
		}
	}

this.checkSystems = function()	
	{
	this.playerDamagedList = [];
	var equipment = player.ship.equipment;
	var listCounter = 0 ; // reset the counter
	for(listCounter = 0;listCounter<equipment.length;listCounter++)
		{
		if(player.ship.equipmentStatus(equipment[listCounter].equipmentKey) !== "EQUIPMENT_DAMAGED")
			{ continue; }
		var scriptEq = EquipmentInfo.infoForKey(equipment[listCounter].equipmentKey);
		if(scriptEq.scriptInfo.thargoidRepairBotChance === undefined 
				|| isNaN(scriptEq.scriptInfo.thargoidRepairBotChance)
				|| (!isNaN(scriptEq.scriptInfo.thargoidRepairBotChance)
				&& scriptEq.scriptInfo.thargoidRepairBotChance > 0)
			)
			{ this.playerDamagedList.push(equipment[listCounter].equipmentKey); } // if it's broke and fixable, add it to the list.
		}
	}
	
this.repairSystems = function() 
	{
	this.checkSystems();
	if(this.playerDamagedList.length === 0)
		{
		player.consoleMessage("Available ship repair completed - Repair system entering standby mode", 5);
		return;
		}
	else
		{
		var damagedEquipment = Math.floor(Math.random() * this.playerDamagedList.length); // pick a random element from the list...
		this.fixedItem = this.playerDamagedList[damagedEquipment]; // ...define the item...
		this.fixedName = EquipmentInfo.infoForKey(this.fixedItem).name; // define it's screen name
		this.fixedTech = EquipmentInfo.infoForKey(this.fixedItem).effectiveTechLevel // tech level of the item

		switch(true) 
			{
			case (EquipmentInfo.infoForKey(this.fixedItem).scriptInfo.thargoidRepairBotChance !== undefined && !isNaN(EquipmentInfo.infoForKey(this.fixedItem).scriptInfo.thargoidRepairBotChance)): 
				{this.fixChance = EquipmentInfo.infoForKey(this.fixedItem).scriptInfo.thargoidRepairBotChance; break;}
			case (this.fixedTech < 9): {this.fixChance = 1; break;}
			case ((this.fixedTech > 8) && (this.fixedTech < 17)): {this.fixChance = 1 - ((this.fixedTech - 8)/10); break;}
			case (this.fixedTech == 99): {this.fixChance = 0.1; break;}
			default: {this.fixChance = 0.2; break;}
			}
		}	
		
	if(Math.random() < this.fixChance)
		{ this.fixItem(); }
	else
		{
		if(missionVariables.repairCounter && missionVariables.repairCounter > 0)
			{
			var repairTime = 120 + Math.ceil(Math.random() * 180);
			player.consoleMessage(this.fixedName + " repair attempt failed - work continuing.", 5);
			this.damageControlTimer = new Timer(this, this.repairSystems, repairTime);
			}
		else	
			{ player.consoleMessage("Repair system exhausted - recharge required", 6); }
		}
	}


this.fixItem = function()
	{
	player.ship.setEquipmentStatus(this.fixedItem,"EQUIPMENT_OK"); // and actually fix the thing!
	player.consoleMessage(this.fixedName + " online and operational", 5)
	switch(this.fixedItem) // specific OXP equipment which need rebooting after fixing, or have other issues.
		{
		case "EQ_FRAME_FUEL_COLLECTOR":
			{
			if(worldScripts["Fuel Collector"]) 
				{
				worldScripts["Fuel Collector"].shipLaunchedFromStation(); // restart the timers in it's world script
				break;
				}
			}
		case "EQ_FRAME_BOUNTY_SCANNER":			
			{
			if(worldScripts["Bounty Scanner"])
				{
				worldScripts["Bounty Scanner"].shipLaunchedFromStation(); // restart the timers in it's world script
				break;
				}
			}
		case "EQ_EEU":
			{
			if(worldScripts["Emergency Energy Unit"])
				{
				worldScripts["Emergency Energy Unit"].shipLaunchedFromStation(); // restart the timers in it's world script
				break;
				}
			}
		case "EQ_ROCKHERMIT_SCANNER": 
			{ 
			if(worldScripts["rockHermit_Locator"]) 
				{ 
				if(worldScripts["rockHermit_Locator"].version === "1.2.3")
					{
					worldScripts["rockHermit_Locator"].shipLaunchedFromStation() // use the inbuild scripting of the OXP to restart it.
					}
				else
					{
					this.beacons = system.shipsWithPrimaryRole("rockbeacof"); // list the inactive beacons (no lights or "R" on compass) 
					if(this.beacons.length > 0) 
						{ 
						for (let i=0; i<this.beacons.length;i++) 
							{ 
							if (oolite.compareVersion('1.73') <= 0) 
								{
								this.beacons[i].position = this.beacons[i].position.multiply(100); //throw the rockbeacof into space... 
								}
							else 
								{
								this.beacons[i].setPosition(this.beacons[i].position.multiply(100)); //throw the rockbeacof into space... 
						            }

							this.beacons[i].explode(); //...and blow it up. 
							} 
						} 
					worldScripts["rockHermit_Locator"].buoy = "rockbeacon" 
					worldScripts["rockHermit_Locator"].addBuoys() // no inactive beacons, so use original code to add active beacons. 
					}
				break; 
				} 
			}
		}
		
	if(this.playerDamagedList.length > 1)	
		{
		if(missionVariables.repairCounter && missionVariables.repairCounter > 0)
			{
			missionVariables.repairCounter--;
			var repairTime = 120 + Math.ceil(Math.random() * 180);
			player.consoleMessage("Further damage detected - repairs continuing", 6);
			this.damageControlTimer = new Timer(this, this.repairSystems, repairTime);	
			}
		else
			{ player.consoleMessage("Repair system exhausted - recharge required", 6); }
		}		
	}